@@ -1,37 +1,29 @@
-/*
-libdmtx - Data Matrix Encoding/Decoding Library
-
-Copyright (C) 2008, 2009 Mike Laughton
-
-This library is free software; you can redistribute it and/or
-modify it under the terms of the GNU Lesser General Public
-License as published by the Free Software Foundation; either
-version 2.1 of the License, or (at your option) any later version.
-
-This library is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-Lesser General Public License for more details.
-
-You should have received a copy of the GNU Lesser General Public
-License along with this library; if not, write to the Free Software
-Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-Contact: mike@dragonflylogic.com
-*/
-
-/* $Id: dmtxdecode.c 764 2009-02-27 00:27:05Z mblaughton $ */
-
 /**
- * @file dmtxdecode.c
- * @brief Decode regions
+ * libdmtx - Data Matrix Encoding/Decoding Library
+ * Copyright 2008, 2009 Mike Laughton. All rights reserved.
+ * Copyright 2009 Mackenzie Straight. All rights reserved.
+ * Copyright 2012-2016 Vadim A. Misbakh-Soloviov. All rights reserved.
+ * Copyright 2016 Tim Zaman. All rights reserved.
+ *
+ * See LICENSE file in the main project directory for full
+ * terms of use and distribution.
+ *
+ * Contact:
+ * Vadim A. Misbakh-Soloviov <dmtx@mva.name>
+ * Mike Laughton <mike@dragonflylogic.com>
+ *
+ * \file dmtxdecode.c
+ * \brief Decode regions
  */
 
 /**
- * @brief  Initialize decode struct with default values
- * @param  img
- * @return Initialized DmtxDecode struct
+ * \brief  Initialize decode struct with default values
+ * \param  img
+ * \return Initialized DmtxDecode struct
  */
+
+#include <stdio.h> // for snprintf
+
 extern DmtxDecode *
 dmtxDecodeCreate(DmtxImage *img, int scale)
 {
@@ -45,19 +37,20 @@ dmtxDecodeCreate(DmtxImage *img, int scale)
    width = dmtxImageGetProp(img, DmtxPropWidth) / scale;
    height = dmtxImageGetProp(img, DmtxPropHeight) / scale;
 
+   dec->fnc1 = DmtxUndefined;
+
    dec->edgeMin = DmtxUndefined;
    dec->edgeMax = DmtxUndefined;
+   dec->scanGap = 1;
    dec->squareDevn = cos(50 * (M_PI/180));
    dec->sizeIdxExpected = DmtxSymbolShapeAuto;
    dec->edgeThresh = 10;
-   dec->scale = scale;
 
-   /* Unscaled values */
-   dec->scanGap = 1;
    dec->xMin = 0;
    dec->xMax = width - 1;
    dec->yMin = 0;
    dec->yMax = height - 1;
+   dec->scale = scale;
 
    dec->cache = (unsigned char *)calloc(width * height, sizeof(unsigned char));
    if(dec->cache == NULL) {
@@ -72,9 +65,9 @@ dmtxDecodeCreate(DmtxImage *img, int scale)
 }
 
 /**
- * @brief  Deinitialize decode struct
- * @param  dec
- * @return void
+ * \brief  Deinitialize decode struct
+ * \param  dec
+ * \return void
  */
 extern DmtxPassFail
 dmtxDecodeDestroy(DmtxDecode **dec)
@@ -93,11 +86,11 @@ dmtxDecodeDestroy(DmtxDecode **dec)
 }
 
 /**
- * @brief  Set decoding behavior property
- * @param  dec
- * @param  prop
- * @param  value
- * @return DmtxPass | DmtxFail
+ * \brief  Set decoding behavior property
+ * \param  dec
+ * \param  prop
+ * \param  value
+ * \return DmtxPass | DmtxFail
  */
 extern DmtxPassFail
 dmtxDecodeSetProp(DmtxDecode *dec, int prop, int value)
@@ -110,7 +103,10 @@ dmtxDecodeSetProp(DmtxDecode *dec, int prop, int value)
          dec->edgeMax = value;
          break;
       case DmtxPropScanGap:
-         dec->scanGap = value;
+         dec->scanGap = value; /* XXX Should this be scaled? */
+         break;
+      case DmtxPropFnc1:
+         dec->fnc1 = value;
          break;
       case DmtxPropSquareDevn:
          dec->squareDevn = cos(value * (M_PI/180.0));
@@ -154,10 +150,10 @@ dmtxDecodeSetProp(DmtxDecode *dec, int prop, int value)
 }
 
 /**
- * @brief  Get decoding behavior property
- * @param  dec
- * @param  prop
- * @return value
+ * \brief  Get decoding behavior property
+ * \param  dec
+ * \param  prop
+ * \return value
  */
 extern int
 dmtxDecodeGetProp(DmtxDecode *dec, int prop)
@@ -169,6 +165,8 @@ dmtxDecodeGetProp(DmtxDecode *dec, int prop)
          return dec->edgeMax;
       case DmtxPropScanGap:
          return dec->scanGap;
+      case DmtxPropFnc1:
+         return dec->fnc1;
       case DmtxPropSquareDevn:
          return (int)(acos(dec->squareDevn) * 180.0/M_PI);
       case DmtxPropSymbolSize:
@@ -197,11 +195,11 @@ dmtxDecodeGetProp(DmtxDecode *dec, int prop)
 }
 
 /**
- * @brief  Returns xxx
- * @param  img
- * @param  Scaled x coordinate
- * @param  Scaled y coordinate
- * @return Scaled pixel offset
+ * \brief  Returns xxx
+ * \param  img
+ * \param  Scaled x coordinate
+ * \param  Scaled y coordinate
+ * \return Scaled pixel offset
  */
 extern unsigned char *
 dmtxDecodeGetCache(DmtxDecode *dec, int x, int y)
@@ -210,6 +208,9 @@ dmtxDecodeGetCache(DmtxDecode *dec, int x, int y)
 
    assert(dec != NULL);
 
+/* if(dec.cacheComplete == DmtxFalse)
+      CacheImage(); */
+
    width = dmtxDecodeGetProp(dec, DmtxPropWidth);
    height = dmtxDecodeGetProp(dec, DmtxPropHeight);
 
@@ -261,19 +262,78 @@ dmtxDecodeGetPixelValue(DmtxDecode *dec, int x, int y, int channel, int *value)
 }
 
 /**
- * @brief  Convert fitted Data Matrix region into a decoded message
- * @param  dec
- * @param  reg
- * @param  fix
- * @return Decoded message
+ * \brief  Fill the region covered by the quadrilateral given by (p0,p1,p2,p3) in the cache.
+ */
+static void
+CacheFillQuad(DmtxDecode *dec, DmtxPixelLoc p0, DmtxPixelLoc p1, DmtxPixelLoc p2, DmtxPixelLoc p3)
+{
+   DmtxBresLine lines[4];
+   DmtxPixelLoc pEmpty = { 0, 0 };
+   unsigned char *cache;
+   int *scanlineMin, *scanlineMax;
+   int minY, maxY, sizeY, posY, posX;
+   int i, idx;
+
+   lines[0] = BresLineInit(p0, p1, pEmpty);
+   lines[1] = BresLineInit(p1, p2, pEmpty);
+   lines[2] = BresLineInit(p2, p3, pEmpty);
+   lines[3] = BresLineInit(p3, p0, pEmpty);
+
+   minY = dec->yMax;
+   maxY = 0;
+
+   minY = min(minY, p0.Y); maxY = max(maxY, p0.Y);
+   minY = min(minY, p1.Y); maxY = max(maxY, p1.Y);
+   minY = min(minY, p2.Y); maxY = max(maxY, p2.Y);
+   minY = min(minY, p3.Y); maxY = max(maxY, p3.Y);
+
+   sizeY = maxY - minY + 1;
+
+   scanlineMin = (int *)malloc(sizeY * sizeof(int));
+   scanlineMax = (int *)calloc(sizeY, sizeof(int));
+
+   assert(scanlineMin); /* XXX handle this better */
+   assert(scanlineMax); /* XXX handle this better */
+
+   for(i = 0; i < sizeY; i++)
+      scanlineMin[i] = dec->xMax;
+
+   for(i = 0; i < 4; i++) {
+      while(lines[i].loc.X != lines[i].loc1.X || lines[i].loc.Y != lines[i].loc1.Y) {
+         idx = lines[i].loc.Y - minY;
+         scanlineMin[idx] = min(scanlineMin[idx], lines[i].loc.X);
+         scanlineMax[idx] = max(scanlineMax[idx], lines[i].loc.X);
+         BresLineStep(lines + i, 1, 0);
+      }
+   }
+
+   for(posY = minY; posY < maxY && posY < dec->yMax; posY++) {
+      idx = posY - minY;
+      for(posX = scanlineMin[idx]; posX < scanlineMax[idx] && posX < dec->xMax; posX++) {
+         cache = dmtxDecodeGetCache(dec, posX, posY);
+         if(cache != NULL)
+            *cache |= 0x80;
+      }
+   }
+
+   free(scanlineMin);
+   free(scanlineMax);
+}
+
+/**
+ * \brief  Convert fitted Data Matrix region into a decoded message
+ * \param  dec
+ * \param  reg
+ * \param  fix
+ * \return Decoded message
  */
 extern DmtxMessage *
 dmtxDecodeMatrixRegion(DmtxDecode *dec, DmtxRegion *reg, int fix)
 {
-   int row, col;
-   unsigned char *cache;
+   //fprintf(stdout, "libdmtx::dmtxDecodeMatrixRegion()\n");
    DmtxMessage *msg;
-   DmtxVector2 p;
+   DmtxVector2 topLeft, topRight, bottomLeft, bottomRight;
+   DmtxPixelLoc pxTopLeft, pxTopRight, pxBottomLeft, pxBottomRight;
 
    msg = dmtxMessageCreate(reg->sizeIdx, DmtxFormatMatrix);
    if(msg == NULL)
@@ -284,42 +344,81 @@ dmtxDecodeMatrixRegion(DmtxDecode *dec, DmtxRegion *reg, int fix)
       return NULL;
    }
 
-   ModulePlacementEcc200(msg->array, msg->code,
-         reg->sizeIdx, DmtxModuleOnRed | DmtxModuleOnGreen | DmtxModuleOnBlue);
+   msg->fnc1 = dec->fnc1;
 
-   if(DecodeCheckErrors(msg->code, reg->sizeIdx, fix) != DmtxPass) {
+   topLeft.X = bottomLeft.X = topLeft.Y = topRight.Y = -0.1;
+   topRight.X = bottomRight.X = bottomLeft.Y = bottomRight.Y = 1.1;
+
+   dmtxMatrix3VMultiplyBy(&topLeft, reg->fit2raw);
+   dmtxMatrix3VMultiplyBy(&topRight, reg->fit2raw);
+   dmtxMatrix3VMultiplyBy(&bottomLeft, reg->fit2raw);
+   dmtxMatrix3VMultiplyBy(&bottomRight, reg->fit2raw);
+
+   pxTopLeft.X = (int)(0.5 + topLeft.X);
+   pxTopLeft.Y = (int)(0.5 + topLeft.Y);
+   pxBottomLeft.X = (int)(0.5 + bottomLeft.X);
+   pxBottomLeft.Y = (int)(0.5 + bottomLeft.Y);
+   pxTopRight.X = (int)(0.5 + topRight.X);
+   pxTopRight.Y = (int)(0.5 + topRight.Y);
+   pxBottomRight.X = (int)(0.5 + bottomRight.X);
+   pxBottomRight.Y = (int)(0.5 + bottomRight.Y);
+
+   CacheFillQuad(dec, pxTopLeft, pxTopRight, pxBottomRight, pxBottomLeft);
+
+   return dmtxDecodePopulatedArray(reg->sizeIdx, msg, fix);
+}
+
+/**
+ * \brief  Ripped out a part of dmtxDecodeMatrixRegion function to this one to parse own array
+ * \param  sizeIdx
+ * \param  msg
+ * \param  fix
+ * \return Decoded message (msg pointer) or NULL in case of failure.
+ * \note You should reaffect msg with the result of this call
+ *       since a NULL result means msg gets freed and should not be used anymore.
+ *       ex: msg = dmtxDecodePopulatedArray(sizeidx, msg, fix);
+ */
+DmtxMessage *
+dmtxDecodePopulatedArray(int sizeIdx, DmtxMessage *msg, int fix)
+{
+   /*
+    * Example msg->array indices for a 12x12 datamatrix.
+    *  also, the 'L' color (usually black) is defined as 'DmtxModuleOnRGB'
+    *
+    * XX    XX    XX    XX    XX    XX   
+    * XX 0   1  2  3  4  5  6  7  8  9 XX 
+    * XX 10 11 12 13 14 15 16 17 18 19 
+    * XX 20 21 22 23 24 25 26 27 28 29 XX
+    * XX 30 31 32 33 34 35 36 37 38 39 
+    * XX 40 41 42 43 44 45 46 47 48 49 XX
+    * XX 50 51 52 53 54 55 56 57 58 59 
+    * XX 60 61 62 63 64 65 66 67 68 69 XX
+    * XX 70 71 72 73 74 75 76 77 78 79 
+    * XX 80 81 82 83 84 85 86 87 88 89 XX
+    * XX 90 91 92 93 94 95 96 97 98 99 
+    * XX XX XX XX XX XX XX XX XX XX XX XX
+    *
+    */
+    
+   ModulePlacementEcc200(msg->array, msg->code, sizeIdx, DmtxModuleOnRed | DmtxModuleOnGreen | DmtxModuleOnBlue);
+
+   if(RsDecode(msg->code, sizeIdx, fix) == DmtxFail){
       dmtxMessageDestroy(&msg);
+      msg = NULL;
       return NULL;
    }
 
-   for(row = dec->yMin; row < dec->yMax; row++) {
-      for(col = dec->xMin; col < dec->xMax; col++) {
-         p.X = col;
-         p.Y = row;
-         dmtxMatrix3VMultiplyBy(&p, reg->raw2fit);
-         /* XXX tighten these boundaries can by accounting for barcode size */
-         if(p.X >= -0.1 && p.X <= 1.1 && p.Y >= -0.1 && p.Y <= 1.1) {
-
-            cache = dmtxDecodeGetCache(dec, col, row);
-            if(cache == NULL)
-               continue;
-            else
-               *cache |= 0x80; /* Mark as visited */
-         }
-      }
-   }
-
-   DecodeDataStream(msg, reg->sizeIdx, NULL);
+   DecodeDataStream(msg, sizeIdx, NULL);
 
    return msg;
 }
 
 /**
- * @brief  Convert fitted Data Mosaic region into a decoded message
- * @param  dec
- * @param  reg
- * @param  fix
- * @return Decoded message
+ * \brief  Convert fitted Data Mosaic region into a decoded message
+ * \param  dec
+ * \param  reg
+ * \param  fix
+ * \return Decoded message
  */
 extern DmtxMessage *
 dmtxDecodeMosaicRegion(DmtxDecode *dec, DmtxRegion *reg, int fix)
@@ -330,6 +429,19 @@ dmtxDecodeMosaicRegion(DmtxDecode *dec, DmtxRegion *reg, int fix)
 
    colorPlane = reg->flowBegin.plane;
 
+   /**
+    * Consider performing a color cube fit here to identify exact RGB of
+    * all 6 "cube-like" corners based on pixels located within region. Then
+    * force each sample pixel to the "cube-like" corner based o which one
+    * is nearest "sqrt(dr^2+dg^2+db^2)" (except sqrt is unnecessary).
+    * colorPlane = reg->flowBegin.plane;
+    *
+    * To find RGB values of primary colors, perform something like a
+    * histogram except instead of going from black to color N, go from
+    * (127,127,127) to color. Use color bins along with distance to
+    * identify value. An additional method will be required to get actual
+    * RGB instead of just a plane in 3D. */
+
    reg->flowBegin.plane = 0; /* kind of a hack */
    rMsg = dmtxDecodeMatrixRegion(dec, reg, fix);
 
@@ -453,502 +565,16 @@ dmtxDecodeCreateDiagnostic(DmtxDecode *dec, int *totalBytes, int *headerBytes, i
 }
 
 /**
- * @brief  Translate encoded data stream into final output
- * @param  msg
- * @param  sizeIdx
- * @param  outputStart
- * @return void
- */
-static void
-DecodeDataStream(DmtxMessage *msg, int sizeIdx, unsigned char *outputStart)
-{
-   DmtxSchemeDecode encScheme;
-   unsigned char *ptr, *dataEnd;
-
-   msg->output = (outputStart == NULL) ? msg->output : outputStart;
-   msg->outputIdx = 0;
-
-   ptr = msg->code;
-   dataEnd = ptr + dmtxGetSymbolAttribute(DmtxSymAttribSymbolDataWords, sizeIdx);
-
-   while(ptr < dataEnd) {
-
-      ptr = NextEncodationScheme(&encScheme, ptr);
-
-      switch(encScheme) {
-         case DmtxSchemeDecodeAsciiStd:
-            ptr = DecodeSchemeAsciiStd(msg, ptr, dataEnd);
-            break;
-
-         case DmtxSchemeDecodeAsciiExt:
-            ptr = DecodeSchemeAsciiExt(msg, ptr);
-            break;
-
-         case DmtxSchemeDecodeC40:
-         case DmtxSchemeDecodeText:
-            ptr = DecodeSchemeC40Text(msg, ptr, dataEnd, encScheme);
-            break;
-
-         case DmtxSchemeDecodeX12:
-            ptr = DecodeSchemeX12(msg, ptr, dataEnd);
-            break;
-
-         case DmtxSchemeDecodeEdifact:
-            ptr = DecodeSchemeEdifact(msg, ptr, dataEnd);
-            break;
-
-         case DmtxSchemeDecodeBase256:
-            ptr = DecodeSchemeBase256(msg, ptr, dataEnd);
-            break;
-      }
-   }
-}
-
-/**
- * @brief  Determine next encodation scheme
- * @param  encScheme
- * @param  ptr
- * @return Pointer to next undecoded codeword
- */
-static unsigned char *
-NextEncodationScheme(DmtxSchemeDecode *encScheme, unsigned char *ptr)
-{
-   switch(*ptr) {
-      case 230:
-         *encScheme = DmtxSchemeDecodeC40;
-         break;
-      case 231:
-         *encScheme = DmtxSchemeDecodeBase256;
-         break;
-      case 235:
-         *encScheme = DmtxSchemeDecodeAsciiExt;
-         break;
-      case 238:
-         *encScheme = DmtxSchemeDecodeX12;
-         break;
-      case 239:
-         *encScheme = DmtxSchemeDecodeText;
-         break;
-      case 240:
-         *encScheme = DmtxSchemeDecodeEdifact;
-         break;
-      default:
-         *encScheme = DmtxSchemeDecodeAsciiStd;
-         return ptr;
-   }
-
-   return ptr + 1;
-}
-
-/**
- *
- *
- */
-static void
-PushOutputWord(DmtxMessage *msg, int value)
-{
-   assert(value >= 0 && value < 256);
-
-   msg->output[msg->outputIdx++] = (unsigned char)value;
-}
-
-/**
- *
- *
- */
-static void
-PushOutputC40TextWord(DmtxMessage *msg, C40TextState *state, int value)
-{
-   assert(value >= 0 && value < 256);
-
-   msg->output[msg->outputIdx] = (unsigned char)value;
-
-   if(state->upperShift == DmtxTrue) {
-      assert(value < 128);
-      msg->output[msg->outputIdx] += 128;
-   }
-
-   msg->outputIdx++;
-
-   state->shift = DmtxC40TextBasicSet;
-   state->upperShift = DmtxFalse;
-}
-
-/**
- * @brief  Decode stream assuming standard ASCII encodation
- * @param  msg
- * @param  ptr
- * @param  dataEnd
- * @return Pointer to next undecoded codeword
- */
-static unsigned char *
-DecodeSchemeAsciiStd(DmtxMessage *msg, unsigned char *ptr, unsigned char *dataEnd)
-{
-   int codeword, digits;
-
-   codeword = (int)(*ptr);
-
-   if(codeword <= 128) {
-      PushOutputWord(msg, codeword - 1);
-   }
-   else if(codeword == 129) {
-      assert(dataEnd >= ptr);
-      assert(dataEnd - ptr <= INT_MAX);
-      msg->padCount = (int)(dataEnd - ptr);
-      return dataEnd;
-   }
-   else if(codeword <= 229) {
-      digits = codeword - 130;
-      PushOutputWord(msg, digits/10 + '0');
-      PushOutputWord(msg, digits - (digits/10)*10 + '0');
-   }
-
-   return ptr + 1;
-}
-
-/**
- * @brief  Decode stream assuming extended ASCII encodation
- * @param  msg
- * @param  ptr
- * @param  dataEnd
- * @return Pointer to next undecoded codeword
- */
-static unsigned char *
-DecodeSchemeAsciiExt(DmtxMessage *msg, unsigned char *ptr)
-{
-   int codeword;
-
-   codeword = (int)(*ptr);
-   PushOutputWord(msg, codeword + 128);
-
-   return ptr + 1;
-}
-
-/**
- * @brief  Decode stream assuming C40 or Text encodation
- * @param  msg
- * @param  ptr
- * @param  dataEnd
- * @param  encScheme
- * @return Pointer to next undecoded codeword
- */
-static unsigned char *
-DecodeSchemeC40Text(DmtxMessage *msg, unsigned char *ptr, unsigned char *dataEnd, DmtxSchemeDecode encScheme)
-{
-   int i;
-   int packed;
-   int c40Values[3];
-   C40TextState state;
-
-   state.shift = DmtxC40TextBasicSet;
-   state.upperShift = DmtxFalse;
-
-   assert(encScheme == DmtxSchemeDecodeC40 || encScheme == DmtxSchemeDecodeText);
-
-   while(ptr < dataEnd) {
-
-      /* FIXME Also check that ptr+1 is safe to access */
-      packed = (*ptr << 8) | *(ptr+1);
-      c40Values[0] = ((packed - 1)/1600);
-      c40Values[1] = ((packed - 1)/40) % 40;
-      c40Values[2] =  (packed - 1) % 40;
-      ptr += 2;
-
-      for(i = 0; i < 3; i++) {
-         if(state.shift == DmtxC40TextBasicSet) { /* Basic set */
-            if(c40Values[i] <= 2) {
-               state.shift = c40Values[i] + 1;
-            }
-            else if(c40Values[i] == 3) {
-               PushOutputC40TextWord(msg, &state, ' ');
-            }
-            else if(c40Values[i] <= 13) {
-               PushOutputC40TextWord(msg, &state, c40Values[i] - 13 + '9'); /* 0-9 */
-            }
-            else if(c40Values[i] <= 39) {
-               if(encScheme == DmtxSchemeDecodeC40) {
-                  PushOutputC40TextWord(msg, &state, c40Values[i] - 39 + 'Z'); /* A-Z */
-               }
-               else if(encScheme == DmtxSchemeDecodeText) {
-                  PushOutputC40TextWord(msg, &state, c40Values[i] - 39 + 'z'); /* a-z */
-               }
-            }
-         }
-         else if(state.shift == DmtxC40TextShift1) { /* Shift 1 set */
-            PushOutputC40TextWord(msg, &state, c40Values[i]); /* ASCII 0 - 31 */
-         }
-         else if(state.shift == DmtxC40TextShift2) { /* Shift 2 set */
-            if(c40Values[i] <= 14) {
-               PushOutputC40TextWord(msg, &state, c40Values[i] + 33); /* ASCII 33 - 47 */
-            }
-            else if(c40Values[i] <= 21) {
-               PushOutputC40TextWord(msg, &state, c40Values[i] + 43); /* ASCII 58 - 64 */
-            }
-            else if(c40Values[i] <= 26) {
-               PushOutputC40TextWord(msg, &state, c40Values[i] + 69); /* ASCII 91 - 95 */
-            }
-            else if(c40Values[i] == 27) {
-               PushOutputC40TextWord(msg, &state, 0x1d); /* FNC1 -- XXX depends on position? */
-            }
-            else if(c40Values[i] == 30) {
-               state.upperShift = DmtxTrue;
-               state.shift = DmtxC40TextBasicSet;
-            }
-         }
-         else if(state.shift == DmtxC40TextShift3) { /* Shift 3 set */
-            if(encScheme == DmtxSchemeDecodeC40) {
-               PushOutputC40TextWord(msg, &state, c40Values[i] + 96);
-            }
-            else if(encScheme == DmtxSchemeDecodeText) {
-               if(c40Values[i] == 0)
-                  PushOutputC40TextWord(msg, &state, c40Values[i] + 96);
-               else if(c40Values[i] <= 26)
-                  PushOutputC40TextWord(msg, &state, c40Values[i] - 26 + 'Z'); /* A-Z */
-               else
-                  PushOutputC40TextWord(msg, &state, c40Values[i] - 31 + 127); /* { | } ~ DEL */
-            }
-         }
-      }
-
-      /* Unlatch if codeword 254 follows 2 codewords in C40/Text encodation */
-      if(*ptr == 254)
-         return ptr + 1;
-
-      /* Unlatch is implied if only one codeword remains */
-      if(dataEnd - ptr == 1)
-         return ptr;
-   }
-
-   return ptr;
-}
-
-/**
- * @brief  Decode stream assuming X12 encodation
- * @param  msg
- * @param  ptr
- * @param  dataEnd
- * @return Pointer to next undecoded codeword
- */
-static unsigned char *
-DecodeSchemeX12(DmtxMessage *msg, unsigned char *ptr, unsigned char *dataEnd)
-{
-   int i;
-   int packed;
-   int x12Values[3];
-
-   while(ptr < dataEnd) {
-
-      /* FIXME Also check that ptr+1 is safe to access */
-      packed = (*ptr << 8) | *(ptr+1);
-      x12Values[0] = ((packed - 1)/1600);
-      x12Values[1] = ((packed - 1)/40) % 40;
-      x12Values[2] =  (packed - 1) % 40;
-      ptr += 2;
-
-      for(i = 0; i < 3; i++) {
-         if(x12Values[i] == 0)
-            PushOutputWord(msg, 13);
-         else if(x12Values[i] == 1)
-            PushOutputWord(msg, 42);
-         else if(x12Values[i] == 2)
-            PushOutputWord(msg, 62);
-         else if(x12Values[i] == 3)
-            PushOutputWord(msg, 32);
-         else if(x12Values[i] <= 13)
-            PushOutputWord(msg, x12Values[i] + 44);
-         else if(x12Values[i] <= 90)
-            PushOutputWord(msg, x12Values[i] + 51);
-      }
-
-      /* Unlatch if codeword 254 follows 2 codewords in C40/Text encodation */
-      if(*ptr == 254)
-         return ptr + 1;
-
-      /* Unlatch is implied if only one codeword remains */
-      if(dataEnd - ptr == 1)
-         return ptr;
-   }
-
-   return ptr;
-}
-
-/**
- * @brief  Decode stream assuming EDIFACT encodation
- * @param  msg
- * @param  ptr
- * @param  dataEnd
- * @return Pointer to next undecoded codeword
- */
-static unsigned char *
-DecodeSchemeEdifact(DmtxMessage *msg, unsigned char *ptr, unsigned char *dataEnd)
-{
-   int i;
-   unsigned char unpacked[4];
-
-   while(ptr < dataEnd) {
-
-      /* FIXME Also check that ptr+2 is safe to access -- shouldn't be a
-         problem because I'm guessing you can guarantee there will always
-         be at least 3 error codewords */
-      unpacked[0] = (*ptr & 0xfc) >> 2;
-      unpacked[1] = (*ptr & 0x03) << 4 | (*(ptr+1) & 0xf0) >> 4;
-      unpacked[2] = (*(ptr+1) & 0x0f) << 2 | (*(ptr+2) & 0xc0) >> 6;
-      unpacked[3] = *(ptr+2) & 0x3f;
-
-      for(i = 0; i < 4; i++) {
-
-         /* Advance input ptr (4th value comes from already-read 3rd byte) */
-         if(i < 3)
-            ptr++;
-
-         /* Test for unlatch condition */
-         if(unpacked[i] == 0x1f) {
-            assert(msg->output[msg->outputIdx] == 0); /* XXX dirty why? */
-            return ptr;
-         }
-
-         PushOutputWord(msg, unpacked[i] ^ (((unpacked[i] & 0x20) ^ 0x20) << 1));
-      }
-
-      /* Unlatch is implied if fewer than 3 codewords remain */
-      if(dataEnd - ptr < 3) {
-         return ptr;
-      }
-   }
-
-   return ptr;
-
-/* XXX the following version should be safer, but requires testing before replacing the old version
-   int bits = 0;
-   int bitCount = 0;
-   int value;
-
-   while(ptr < dataEnd) {
-
-      if(bitCount < 6) {
-         bits = (bits << 8) | *(ptr++);
-         bitCount += 8;
-      }
-
-      value = bits >> (bitCount - 6);
-      bits -= (value << (bitCount - 6));
-      bitCount -= 6;
-
-      if(value == 0x1f) {
-         assert(bits == 0); // should be padded with zero-value bits
-         return ptr;
-      }
-      PushOutputWord(msg, value ^ (((value & 0x20) ^ 0x20) << 1));
-
-      // Unlatch implied if just completed triplet and 1 or 2 words are left
-      if(bitCount == 0 && dataEnd - ptr - 1 > 0 && dataEnd - ptr - 1 < 3)
-         return ptr;
-   }
-
-   assert(bits == 0); // should be padded with zero-value bits
-   assert(bitCount == 0); // should be padded with zero-value bits
-   return ptr;
-*/
-}
-
-/**
- * @brief  Decode stream assuming Base 256 encodation
- * @param  msg
- * @param  ptr
- * @param  dataEnd
- * @return Pointer to next undecoded codeword
- */
-static unsigned char *
-DecodeSchemeBase256(DmtxMessage *msg, unsigned char *ptr, unsigned char *dataEnd)
-{
-   int d0, d1;
-   int idx;
-   unsigned char *ptrEnd;
-
-   /* Find positional index used for unrandomizing */
-   assert(ptr + 1 >= msg->code);
-   assert(ptr + 1 - msg->code <= INT_MAX);
-   idx = (int)(ptr + 1 - msg->code);
-
-   d0 = UnRandomize255State(*(ptr++), idx++);
-   if(d0 == 0) {
-      ptrEnd = dataEnd;
-   }
-   else if(d0 <= 249) {
-      ptrEnd = ptr + d0;
-   }
-   else {
-      d1 = UnRandomize255State(*(ptr++), idx++);
-      ptrEnd = ptr + (d0 - 249) * 250 + d1;
-   }
-
-   if(ptrEnd > dataEnd) {
-      exit(40); /* XXX needs cleaner error handling */
-   }
-
-   while(ptr < ptrEnd) {
-      PushOutputWord(msg, UnRandomize255State(*(ptr++), idx++));
-   }
-
-   return ptr;
-}
-
-/**
- * @brief  Unrandomize 253 state
- * @param  codewordValue
- * @return codewordPosition
- */
-/**
-static unsigned char
-UnRandomize253State(unsigned char codewordValue, int codewordPosition)
-{
-   int pseudoRandom;
-   int tmp;
-
-   pseudoRandom = ((149 * codewordPosition) % 253) + 1;
-   tmp = codewordValue - pseudoRandom;
-   if(tmp < 1)
-      tmp += 254;
-
-   assert(tmp >= 0 && tmp < 256);
-
-   return (unsigned char)tmp;
-}
-*/
-
-/**
- * @brief  Unrandomize 255 state
- * @param  value
- * @param  idx
- * @return Unrandomized value
- */
-static unsigned char
-UnRandomize255State(unsigned char value, int idx)
-{
-   int pseudoRandom;
-   int tmp;
-
-   pseudoRandom = ((149 * idx) % 255) + 1;
-   tmp = value - pseudoRandom;
-   if(tmp < 0)
-      tmp += 256;
-
-   assert(tmp >= 0 && tmp < 256);
-
-   return (unsigned char)tmp;
-}
-
-/**
- * @brief  Increment counters used to determine module values
- * @param  img
- * @param  reg
- * @param  tally
- * @param  xOrigin
- * @param  yOrigin
- * @param  mapWidth
- * @param  mapHeight
- * @param  dir
- * @return void
+ * \brief  Increment counters used to determine module values
+ * \param  img
+ * \param  reg
+ * \param  tally
+ * \param  xOrigin
+ * \param  yOrigin
+ * \param  mapWidth
+ * \param  mapHeight
+ * \param  dir
+ * \return void
  */
 static void
 TallyModuleJumps(DmtxDecode *dec, DmtxRegion *reg, int tally[][24], int xOrigin, int yOrigin, int mapWidth, int mapHeight, DmtxDirection dir)
@@ -1004,6 +630,8 @@ TallyModuleJumps(DmtxDecode *dec, DmtxRegion *reg, int tally[][24], int xOrigin,
       /* Capture tModule for each leading border module as normal but
          decide status based on predictable barcode border pattern */
 
+
+
       *travel = travelStart;
       color = ReadModuleColor(dec, reg, symbolRow, symbolCol, reg->sizeIdx, reg->flowBegin.plane);
       tModule = (darkOnLight) ? reg->offColor - color : color - reg->offColor;
@@ -1024,24 +652,27 @@ TallyModuleJumps(DmtxDecode *dec, DmtxRegion *reg, int tally[][24], int xOrigin,
          tModule = (darkOnLight) ? reg->offColor - color : color - reg->offColor;
 
          if(statusPrev == DmtxModuleOnRGB) {
-            if(tModule < tPrev - jumpThreshold)
+            if(tModule < tPrev - jumpThreshold){
                statusModule = DmtxModuleOff;
-            else
+            } else {
                statusModule = DmtxModuleOnRGB;
+            }
          }
          else if(statusPrev == DmtxModuleOff) {
-            if(tModule > tPrev + jumpThreshold)
+            if(tModule > tPrev + jumpThreshold) {
                statusModule = DmtxModuleOnRGB;
-            else
+            } else {
                statusModule = DmtxModuleOff;
+            }
          }
 
          mapRow = symbolRow - yOrigin;
          mapCol = symbolCol - xOrigin;
          assert(mapRow < 24 && mapCol < 24);
 
-         if(statusModule == DmtxModuleOnRGB)
+         if(statusModule == DmtxModuleOnRGB){
             tally[mapRow][mapCol] += (2 * weight);
+         }
 
          weight--;
       }
@@ -1051,15 +682,16 @@ TallyModuleJumps(DmtxDecode *dec, DmtxRegion *reg, int tally[][24], int xOrigin,
 }
 
 /**
- * @brief  Populate array with codeword values based on module colors
- * @param  msg
- * @param  img
- * @param  reg
- * @return DmtxPass | DmtxFail
+ * \brief  Populate array with codeword values based on module colors
+ * \param  msg
+ * \param  img
+ * \param  reg
+ * \return DmtxPass | DmtxFail
  */
 static DmtxPassFail
 PopulateArrayFromMatrix(DmtxDecode *dec, DmtxRegion *reg, DmtxMessage *msg)
 {
+   //fprintf(stdout, "libdmtx::PopulateArrayFromMatrix()\n");
    int weightFactor;
    int mapWidth, mapHeight;
    int xRegionTotal, yRegionTotal;
@@ -1082,6 +714,27 @@ PopulateArrayFromMatrix(DmtxDecode *dec, DmtxRegion *reg, DmtxMessage *msg)
    weightFactor = 2 * (mapHeight + mapWidth + 2);
    assert(weightFactor > 0);
 
+   //fprintf(stdout, "libdmtx::PopulateArrayFromMatrix::reg->sizeIdx: %d\n", reg->sizeIdx);
+   //fprintf(stdout, "libdmtx::PopulateArrayFromMatrix::reg->flowBegin.plane: %d\n", reg->flowBegin.plane);
+   //fprintf(stdout, "libdmtx::PopulateArrayFromMatrix::reg->onColor: %d\n", reg->onColor);
+   //fprintf(stdout, "libdmtx::PopulateArrayFromMatrix::reg->offColor: %d\n", reg->offColor);
+   //fprintf(stdout, "libdmtx::PopulateArrayFromMatrix::xRegionTotal: %d\n", xRegionTotal);
+   //fprintf(stdout, "libdmtx::PopulateArrayFromMatrix::yRegionTotal: %d\n", yRegionTotal);
+   //fprintf(stdout, "libdmtx::PopulateArrayFromMatrix::mapWidth: %d\n", mapWidth);
+   //fprintf(stdout, "libdmtx::PopulateArrayFromMatrix::mapHeight: %d\n", mapHeight);
+   //fprintf(stdout, "libdmtx::PopulateArrayFromMatrix::weightFactor: %d\n", weightFactor);
+   //reg->fit2raw[1][0]=0;
+   //reg->fit2raw[0][1]=0;
+   //reg->fit2raw[0][2]=0;
+   //reg->fit2raw[2][2]=1;
+   //reg->fit2raw[1][2]=0;
+   //reg->fit2raw[2][0]=10; //translation
+   //reg->fit2raw[2][1]=10; //translation
+   //reg->fit2raw[0][0]=60; //scale
+   //reg->fit2raw[1][1]=60; //scale
+   //dmtxMatrix3Print(reg->fit2raw);
+   
+
    /* Tally module changes for each region in each direction */
    for(yRegionCount = 0; yRegionCount < yRegionTotal; yRegionCount++) {
 
@@ -1092,6 +745,7 @@ PopulateArrayFromMatrix(DmtxDecode *dec, DmtxRegion *reg, DmtxMessage *msg)
 
          /* X location of mapping region origin in symbol coordinates */
          xOrigin = xRegionCount * (mapWidth + 2) + 1;
+         //fprintf(stdout, "libdmtx::PopulateArrayFromMatrix::xOrigin: %d\n", xOrigin);
 
          memset(tally, 0x00, 24 * 24 * sizeof(int));
          TallyModuleJumps(dec, reg, tally, xOrigin, yOrigin, mapWidth, mapHeight, DmtxDirUp);
@@ -1101,20 +755,26 @@ PopulateArrayFromMatrix(DmtxDecode *dec, DmtxRegion *reg, DmtxMessage *msg)
 
          /* Decide module status based on final tallies */
          for(mapRow = 0; mapRow < mapHeight; mapRow++) {
+         //for(mapRow = mapHeight-1; mapRow >= 0; mapRow--) {
             for(mapCol = 0; mapCol < mapWidth; mapCol++) {
-
+               
                rowTmp = (yRegionCount * mapHeight) + mapRow;
                rowTmp = yRegionTotal * mapHeight - rowTmp - 1;
                colTmp = (xRegionCount * mapWidth) + mapCol;
                idx = (rowTmp * xRegionTotal * mapWidth) + colTmp;
-
-               if(tally[mapRow][mapCol]/(double)weightFactor >= 0.5)
+               //fprintf(stdout, "libdmtx::PopulateArrayFromMatrix::idx: %d @ %d,%d\n", idx, mapCol, mapRow);
+               //fprintf(stdout, "%c ",tally[mapRow][mapCol]==DmtxModuleOff ? 'X' : ' ');
+               if(tally[mapRow][mapCol]/(double)weightFactor >= 0.5){
                   msg->array[idx] = DmtxModuleOnRGB;
-               else
+                  //fprintf(stdout, "X ");
+               } else {
                   msg->array[idx] = DmtxModuleOff;
+                  //fprintf(stdout, "  ");
+               }
 
                msg->array[idx] |= DmtxModuleAssigned;
             }
+            //fprintf(stdout, "\n");
          }
       }
    }
