@@ -11,12 +11,7 @@ function RunTest()
    for dir in $(find "$LIBDMTX" -type d); do
 
       if [[ "$dir" != "$LIBDMTX" &&
-            "$dir" != "$LIBDMTX/util/dmtxread" &&
-            "$dir" != "$LIBDMTX/util/dmtxwrite" &&
-            "$dir" != "$LIBDMTX/test/rotate_test" &&
-            "$dir" != "$LIBDMTX/test/simple_test" &&
-            "$dir" != "$LIBDMTX/test/unit_test" &&
-            "$dir" != "$LIBDMTX/test/script" ]]; then
+            "$dir" != "$LIBDMTX/test/simple_test" ]]; then
          continue
       fi
 
@@ -34,7 +29,7 @@ function RunTest()
          fi
 
          if [[ $(cat $file | wc -l) -le 10 ]]; then
-            echo "      skipping \"$file\" (trivial file)"
+            #echo "      skipping \"$file\" (trivial file)"
             continue
          fi
 
@@ -62,13 +57,12 @@ if [[ -z "$LIBDMTX" || ! -d "$LIBDMTX/script" ]]; then
    exit 1
 fi
 
-RunTest check_comments.sh || exit 1
-RunTest check_copyright.sh || exit 1
-RunTest check_keyword.sh || exit 1
-RunTest check_license.sh || exit 1
-RunTest check_spacing.sh || exit 1
-RunTest check_whitespace.sh || exit 1
-RunTest check_headers.pl || exit 1
+RunTest check_comments.sh
+RunTest check_copyright.sh
+RunTest check_license.sh
+RunTest check_spacing.sh
+RunTest check_whitespace.sh
+RunTest check_headers.pl
 RunTest check_todo.sh
 
 exit 0
